/*
 * T-Plan Robot Enterprise, automated testing tool.
 * Copyright (C) 2016 T-Plan Limited (http://www.t-plan.com),
 * All rights reserved.
 */
package com.tplan.extent;

import com.tplan.robot.preferences.Preference;
import com.tplan.robot.scripting.DefaultJavaTestScript;
import com.tplan.robot.scripting.ParametrizedTestScript;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * Starts the Extent report from a T-Plan Robot test script.
 */
public class Report extends DefaultJavaTestScript implements ParametrizedTestScript {

    // Parameter "file" - path to the report HTML file
    public static final String PARAM_FILE = "file";
    // Parameter "name" - test name
    public static final String PARAM_NAME = "name";
    // Parameter "desc" - test description
    public static final String PARAM_DESC = "desc";
    // Parameter "logs" - flag to enable or disable log recording
    public static final String PARAM_LOGS = "logs";
    // Parameter "project" - the project name used for report categorization (since v0.3)
    public static final String PARAM_PROJECT = "project";
    // Parameter "extentx" - address of the ExtentX server (since v0.3)
    public static final String PARAM_EXTENTX = "extentx";

    // Parameter "extentx" - address of the Klov server (since v0.4)
    public static final String PARAM_KLOV = "klov";

    // Default Extent report file name
    public static final String DEFAULT_REPORT_FILE_NAME = "extentreport.html";

    @Override
    public void test() {
        // Read the script parameters from the context
        final String file = getVariableAsString(PARAM_FILE);
        final String name = getVariableAsString(PARAM_NAME);
        final String desc = getVariableAsString(PARAM_DESC);
        final String project = getVariableAsString(PARAM_PROJECT);

        // ExtentX integration supported since v0.3
        String mongoAddress = getVariableAsString(PARAM_EXTENTX);
        String host = MongoDBTester.parseServer(mongoAddress);
        int port;
        try {
            port = MongoDBTester.parsePort(mongoAddress, MongoDBTester.MONGODB_DEFAULT_PORT);
        } catch (IllegalArgumentException ile) {
            String err = "Illegal ExtentX/MongoDB address of '" + mongoAddress + "': " + ile.getMessage();
            getContext().getExecutionLog().logSevere(err, false);
            throw new IllegalArgumentException(err);
        }

        String klovAddress = getVariableAsString(PARAM_KLOV);
        String klovHost = MongoDBTester.parseServer(klovAddress);
        int klovPort;
        try {
            klovPort = MongoDBTester.parsePort(klovAddress, MongoDBTester.KLOV_DEFAULT_PORT);
        } catch (IllegalArgumentException ile) {
            String err = "Illegal Klov address of '" + klovAddress + "': " + ile.getMessage();
            getContext().getExecutionLog().logSevere(err, false);
            throw new IllegalArgumentException(err);
        }

        // Enable log recording - disabled by default
        Boolean b = getVariableAsBoolean(PARAM_LOGS);
        final boolean recordLogs = b == null ? false : b;

        // Create an Extent report and start it
        try {
            ExtentReport report = ExtentReportFactory.getInstance().getReportInstance();
            report.startReport(getContext(), file, name, desc, project, host, port, klovHost, klovPort, recordLogs);
        } catch (Throwable t) {
            t.printStackTrace();
            getContext().getExecutionLog().logSevere("Failed to create the Extent report for "
                    + t.getClass().getName() + ": " + t.getMessage(), false);
        }
    }

    @Override
    public String getShortDescription() {
        return "Create an Extent report.";
    }

    @Override
    public String getLongDescription() {
        return getShortDescription();
    }

    @Override
    public List<String> getParameters() {
        return Arrays.asList(new String[]{PARAM_FILE, PARAM_NAME, PARAM_DESC, PARAM_LOGS, PARAM_PROJECT, PARAM_EXTENTX, PARAM_KLOV});
    }

    @Override
    public List getParameterValues(String parameterName) {
        if (PARAM_LOGS.equalsIgnoreCase(parameterName)) {
            return Arrays.asList(new String[]{"true", "false"});
        }
        // None of the other parameters is the "one of" value type
        return null;
    }

    @Override
    public void setParameters(Map<String, String> paramsAndValues) {
        // TODO: Validate parameters
    }

    @Override
    public List<Preference> getVisualParameters() {
        List<Preference> l = new ArrayList<Preference>();
        String container = "Extent Report Properties";
        String desc = "<html><body>"
                + "Start an Extent HTML report. Read the "
                + "<a href=\"http://www.t-plan.com/robot/docs/plugins/extentreports.html#report\">plugin docs</a> for details."
                + "</body></html>";
        Preference p = new Preference(PARAM_FILE, Preference.TYPE_FILE, "Target HTML file", desc);
        p.setMandatory(true);
        p.setAcceptEmptyValue(false);
        p.setDefaultValue(DEFAULT_REPORT_FILE_NAME);
        p.setPreferredContainerName(container);
        l.add(p);

        p = new Preference(PARAM_NAME, Preference.TYPE_STRING, "Test name", null);
        p.setPreferredContainerName(container);
        p.setDefaultValue("", false);
        l.add(p);
        p = new Preference(PARAM_DESC, Preference.TYPE_TEXT, "Test description", null);
        p.setDefaultValue("", false);
        p.setTextRowsToDisplay(3);
        p.setPreferredContainerName(container);
        l.add(p);
        p = new Preference(PARAM_LOGS, Preference.TYPE_BOOLEAN, "Record Robot logs too", null);
        p.setDefaultValue(false, false);
        p.setPreferredContainerName(container);
        l.add(p);

        // ExtentX parameters supported since v0.3
        boolean legacyMode = false;
        try {
            ExtentReport e = ExtentReportFactory.getInstance().getReportInstance();
            legacyMode = "ExtentReportImpl".equals(e.getClass().getSimpleName());
        } catch (Exception ex) {
            // Not interested
        }

        if (legacyMode) {
            container = "ExtentX Server Integration";
            desc = "<html><body>"
                    + "The report may be optionally uploaded to the "
                    + "<a href=\"http://extentreports.relevantcodes.com/extentx\">ExtentX dashboard</a>."
                    + "</body></html>";
            p = new Preference(PARAM_EXTENTX, Preference.TYPE_STRING_WITH_ACTION, "ExtentX/MongoDB server[:port]", desc);
            p.setPreferredContainerName(container);
            p.setComponentManager(new MongoDBTester());
            p.setDefaultValue("", false);
            l.add(p);
        } else {
            container = "Klov Server Integration";
            desc = "<html><body>"
                    + "The report may be optionally uploaded to the "
                    + "<a href=\"http://extentreports.com/docs/klov\">Klov dashboard</a>.<br/>"
                    + "You must provide address and port of the Mongo database and the Klov server.<br/>"
                    + "For a typical local installation the addresses are <code>localhost:"+MongoDBTester.MONGODB_DEFAULT_PORT
                    + "</code> and <br/><code>localhost:"+MongoDBTester.KLOV_DEFAULT_PORT+"</code>."
                    + "</body></html>";
            p = new Preference(PARAM_EXTENTX, Preference.TYPE_STRING_WITH_ACTION, "MongoDB server[:port]", desc);
            p.setPreferredContainerName(container);
            p.setComponentManager(new MongoDBTester());
            p.setDefaultValue("", false);
            l.add(p);

            p = new Preference(PARAM_KLOV, Preference.TYPE_STRING_WITH_ACTION, "Klov server[:port]", null);
            p.setPreferredContainerName(container);
            p.setComponentManager(new MongoDBTester());
            p.setDefaultValue("", false);
            l.add(p);
        }

        p = new Preference(PARAM_PROJECT, Preference.TYPE_STRING, "Project name (category)", null);
        p.setPreferredContainerName(container);
        p.setDefaultValue("", false);
        l.add(p);

        return l;
    }
}
