/*
 * T-Plan Robot Enterprise, automated testing tool.
 * Copyright (C) 2016 T-Plan Limited (http://www.t-plan.com),
 * All rights reserved.
 */
package com.tplan.extent;

import com.tplan.robot.preferences.Preference;
import com.tplan.robot.scripting.DefaultJavaTestScript;
import com.tplan.robot.scripting.ParametrizedTestScript;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * Set attributes of the current Extent report.
 */
public class Set extends DefaultJavaTestScript implements ParametrizedTestScript {

    // Parameter "author" - add test author
    public static final String PARAM_AUTHOR = "author";
    // Parameter "param" - system info param name
    public static final String PARAM_PARAM = "param";
    // Parameter "value" - system info param value
    public static final String PARAM_VALUE = "value";
    // Parameter "category" - test category
    public static final String PARAM_CATEGORY = "category";
    // Parameter "config" - configuration file
    public static final String PARAM_CONFIG = "config";

    @Override
    public void test() {
        // Read the script parameters from the context
        final String author = getVariableAsString(PARAM_AUTHOR);
        final String param = getVariableAsString(PARAM_PARAM);
        final String value = getVariableAsString(PARAM_VALUE);
        final String category = getVariableAsString(PARAM_CATEGORY);
        final String config = getVariableAsString(PARAM_CONFIG);
        
        // Look in the script context if there are any previously created 
        // Extent report generators and set their parameters
        Object o = getContext().get(ExtentReport.CONTEXT_KEY_EXTENT_REPORT_LIST);
        if (o != null && o instanceof List) {
            List<ExtentReport> reports = (List<ExtentReport>)o;
            for (ExtentReport report : reports) {
                if (author != null && !author.trim().isEmpty()) {
                    report.addAuthor(author);
                }
                if (param != null && !param.trim().isEmpty() && value != null) {
                    report.addSystemInfo(param, value);
                }
                if (category != null && !category.trim().isEmpty()) {
                    report.addCategory(category);
                }
                if (config != null) {
                    File f = new File(config);
                    if (f.canRead()) {
                        report.setConfiguration(f);
                    }
                }
            }
        }        
    }
    @Override
    public String getShortDescription() {
        return "Set the Extent report attributes.";
    }

    @Override
    public String getLongDescription() {
        return getShortDescription();
    }

    @Override
    public List<String> getParameters() {
        return Arrays.asList(new String[] {PARAM_AUTHOR, PARAM_VALUE, PARAM_PARAM, PARAM_CATEGORY, PARAM_CONFIG});
    }

    @Override
    public List getParameterValues(String parameterName) {
        // None of the parameters is the "one of" value type
        return null;
    }

    @Override
    public void setParameters(Map<String, String> paramsAndValues) {
        // TODO: Validate
    }

    @Override
    public List<Preference> getVisualParameters() {
        List<Preference> l = new ArrayList<Preference>();
        final String container = "Set Report Attributes";
        String desc = "<html><body>"
                + "Set the Extent report attributes such as the author, system info and test category.<br/>"
                + "Read the <a href=\"http://www.t-plan.com/robot/docs/plugins/extentreports.html#set\">Set</a> docs for details."
                + "</body></html>";
        Preference p = new Preference(PARAM_AUTHOR, Preference.TYPE_STRING, "Author name", desc);
        p.setPreferredContainerName(container);
        l.add(p);
        p = new Preference(PARAM_CATEGORY, Preference.TYPE_STRING, "Test category", null);
        p.setDefaultValue("", false);
        p.setPreferredContainerName(container);
        l.add(p);
        p = new Preference(PARAM_PARAM, Preference.TYPE_STRING, "System info - name", null);
        p.setDefaultValue("", false);
        p.setPreferredContainerName(container);
        l.add(p);
        p = new Preference(PARAM_VALUE, Preference.TYPE_STRING, "System info - value", null);
        p.setDefaultValue("", false);
        p.setPreferredContainerName(container);
        l.add(p);
        p = new Preference(PARAM_CONFIG, Preference.TYPE_FILE, "Configuration file", null);
        p.setDefaultValue("", false);
        p.setPreferredContainerName(container);
        l.add(p);
        return l;
    }
}
